@@ -25,8 +25,10 @@ import platform
 import subprocess
 import sys
 import traceback
+import uuid
 
 from pathlib import Path
+from textwrap import dedent
 from urllib.parse import quote_plus
 from xml.sax.saxutils import escape as xml_escape
 
@@ -46,7 +48,10 @@ from gi.repository import Vte
 
 import cairo
 
+from guake import gtk_version
+from guake import guake_version
 from guake import notifier
+from guake import vte_version
 from guake.about import AboutDialog
 from guake.common import gladefile
 from guake.common import pixmapfile
@@ -56,20 +61,21 @@ from guake.globals import ALIGN_LEFT
 from guake.globals import ALIGN_RIGHT
 from guake.globals import ALIGN_TOP
 from guake.globals import ALWAYS_ON_PRIMARY
-from guake.globals import LOCALE_DIR
 from guake.globals import NAME
-from guake.globals import SCHEMA_DIR
-from guake.globals import _
 from guake.gsettings import GSettingHandler
 from guake.guake_logging import setupLogging
 from guake.guake_notebook import GuakeNotebook
 from guake.keybindings import Keybindings
+from guake.paths import LOCALE_DIR
+from guake.paths import SCHEMA_DIR
 from guake.prefs import PrefsDialog
+from guake.prefs import refresh_user_start
 from guake.settings import Settings
 from guake.simplegladeapp import SimpleGladeApp
-from guake.simplegladeapp import bindtextdomain
 from guake.terminal import GuakeTerminalBox
+from guake.theme import select_gtk_theme
 from guake.utils import get_server_time
+from locale import gettext as _
 
 libutempter = None
 try:
@@ -98,9 +104,6 @@ enable_find = False
 
 GObject.threads_init()
 
-# Loading translation
-bindtextdomain(NAME, LOCALE_DIR)
-
 # Setting gobject program name
 GObject.set_prgname(NAME)
 
@@ -162,12 +165,12 @@ class Guake(SimpleGladeApp):
         self.settings = Settings(schema_source)
         self.debug_mode = self.settings.general.get_boolean('debug-mode')
         setupLogging(self.debug_mode)
+        log.info('Guake Terminal %s', guake_version())
+        log.info('VTE %s', vte_version())
+        log.info('Gtk %s', gtk_version())
 
-        self._select_gtk_theme()
-
-        # Cannot use "getattr(gtk.Window().get_style(), "base")[int(gtk.STATE_SELECTED)]"
-        # since theme has not been applied before first show_all
-        self.selected_color = None
+        select_gtk_theme(self.settings)
+        self._patch_theme()
 
         self.prompt_dialog = None
         self.hidden = True
@@ -387,6 +390,8 @@ class Guake(SimpleGladeApp):
         if self.settings.general.get_boolean('start-fullscreen'):
             self.fullscreen()
 
+        refresh_user_start(self.settings)
+
         # Pop-up that shows that guake is working properly (if not
         # unset in the preferences windows)
         if self.settings.general.get_boolean('use-popup'):
@@ -402,17 +407,29 @@ class Guake(SimpleGladeApp):
 
         log.info("Guake initialized")
 
-    def _select_gtk_theme(self):
-        gtk_theme_name = self.settings.general.get_string('gtk-theme-name')
-        log.debug("Wanted GTK theme: %r", gtk_theme_name)
-        settings = Gtk.Settings.get_default()
-        if (Path("/usr/share/themes") / gtk_theme_name).exists():
-            settings.set_property("gtk-theme-name", gtk_theme_name)
+    def _patch_theme(self):
+        style_provider = Gtk.CssProvider()
+        win = Gtk.Window()
+        style_context = win.get_style_context()
 
-        prefer_dark_theme = self.settings.general.get_boolean('gtk-prefer-dark-theme')
-        log.debug("Prefer dark theme: %r", prefer_dark_theme)
-        if prefer_dark_theme:
-            settings.set_property("gtk-application-prefer-dark-theme", prefer_dark_theme)
+        def rgba_to_hex(color):
+            return "#{0:02x}{1:02x}{2:02x}".format(
+                int(color.red * 255), int(color.green * 255), int(color.blue * 255)
+            )
+
+        selected_bg_color = rgba_to_hex(style_context.lookup_color('theme_selected_bg_color')[1])
+        css_data = dedent(
+            """
+            .tab:checked {{
+              color: white;
+              background-color: {};
+            }}
+            """.format(selected_bg_color)
+        ).encode()
+        style_provider.load_from_data(css_data)
+        Gtk.StyleContext.add_provider_for_screen(
+            Gdk.Screen.get_default(), style_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
+        )
 
     # load the custom commands infrastucture
     def load_custom_commands(self):
@@ -421,7 +438,7 @@ class Guake(SimpleGladeApp):
         custom_commands_menu = Gtk.Menu()
         if not self.get_custom_commands(custom_commands_menu):
             return
-        menu = Gtk.MenuItem("Custom Commands")
+        menu = Gtk.MenuItem(_("Custom Commands"))
         menu.set_submenu(custom_commands_menu)
         menu.show()
         self.custom_command_menuitem = menu
@@ -492,7 +509,7 @@ class Guake(SimpleGladeApp):
 
     # new color methods should be moved to the GuakeTerminal class
 
-    def __load_palette(self):
+    def _load_palette(self):
         colorRGBA = Gdk.RGBA(0, 0, 0, 0)
         paletteList = list()
         for color in self.settings.styleFont.get_string("palette").split(':'):
@@ -500,7 +517,7 @@ class Guake(SimpleGladeApp):
             paletteList.append(colorRGBA.copy())
         return paletteList
 
-    def __get_background_color(self, palette_list, transparency):
+    def _get_background_color(self, palette_list, transparency):
         if len(palette_list) > 16:
             bg_color = palette_list[17]
         else:
@@ -517,8 +534,8 @@ class Guake(SimpleGladeApp):
 
     def set_colors_from_settings(self):
         transparency = self.settings.styleBackground.get_int('transparency')
-        palette_list = self.__load_palette()
-        bg_color = self.__get_background_color(palette_list, transparency)
+        palette_list = self._load_palette()
+        bg_color = self._get_background_color(palette_list, transparency)
 
         if len(palette_list) > 16:
             font_color = palette_list[16]
@@ -545,7 +562,7 @@ class Guake(SimpleGladeApp):
         index = self.notebook.get_current_page()
         index = tab or self.notebook.get_current_page()
         for terminal in self.notebook.get_terminals_for_tab(index):
-            terminal.feed_child(command, len(command))
+            terminal.feed_child(command)
             break
 
     def execute_command_by_uuid(self, tab_uuid, command):
@@ -565,7 +582,7 @@ class Guake(SimpleGladeApp):
         else:
             terminals = self.notebook.get_terminals_for_tab(tab_index)
             for current_vte in terminals:
-                current_vte.feed_child(command, len(command))
+                current_vte.feed_child(command)
 
     # TODO this is dead: 2eae380b1a91a24f6c1eb68c13dac33db98a6ea2 and
     # 3f8c344519c9228deb9ca5f181cbdd5ef1d6acc0
@@ -622,8 +639,8 @@ class Guake(SimpleGladeApp):
 
         value = self.settings.general.get_boolean('window-losefocus')
         visible = window.get_property('visible')
+        self.losefocus_time = get_server_time(self.window)
         if visible and value:
-            self.losefocus_time = get_server_time(self.window)
             log.info("Hiding on focus lose")
             self.hide()
 
@@ -641,6 +658,16 @@ class Guake(SimpleGladeApp):
         if event.button != 3:
             return False
 
+        if event.get_state() & Gdk.ModifierType.SHIFT_MASK:
+            # Force showing contextual menu on Ctrl+right click
+            self.showing_context_menu = True
+        else:
+            # First send to background process if handled, do nothing else
+            if Vte.Terminal.do_button_press_event(terminal, event):
+                log.info("Background app captured the right click event")
+                return True
+
+        log.debug("showing context menu")
         self.showing_context_menu = True
 
         guake_clipboard = Gtk.Clipboard.get_default(self.window.get_display())
@@ -671,14 +698,33 @@ class Guake(SimpleGladeApp):
             self.get_widget('context_search_on_web').set_label(_("Search on Web (no selection)"))
             self.get_widget('context_search_on_web').set_visible(False)
 
-        link = self.getCurrentTerminalLinkUnderCursor()
+        filename = None
+        if current_selection:
+            filename = self.get_current_terminal_filename_under_cursor(current_selection)
+
+        if filename:
+            self.get_widget('context_quick_open').set_visible(True)
+            filename_str = str(filename)
+            if len(filename_str) >= 28:
+                self.get_widget('context_quick_open'
+                                ).set_label(_("Quick Open: {!s}...").format(filename_str[:25]))
+            else:
+                self.get_widget('context_quick_open'
+                                ).set_label(_("Quick Open: {!s}").format(filename_str))
+            self.get_widget('separator_search').set_visible(True)
+        else:
+            self.get_widget('context_quick_open').set_label(_("Quick Open..."))
+            self.get_widget('context_quick_open').set_visible(False)
+
+        link = self.get_current_terminal_link_under_cursor()
         if link:
             self.get_widget('context_browse_on_web').set_visible(True)
             if len(link) >= 28:
                 self.get_widget('context_browse_on_web'
-                                ).set_label(_("Open Link: '{}...'".format(link[:25])))
+                                ).set_label(_("Open Link: {!s}...").format(link[:25]))
             else:
-                self.get_widget('context_browse_on_web').set_label(_("Open Link: {}".format(link)))
+                self.get_widget('context_browse_on_web'
+                                ).set_label(_("Open Link: {!s}").format(link))
             self.get_widget('separator_search').set_visible(True)
         else:
             self.get_widget('context_browse_on_web').set_label(_("Open Link..."))
@@ -865,18 +911,6 @@ class Guake(SimpleGladeApp):
         # thing which includes the scrollbar too
         self.settings.general.triggerOnChangedValue(self.settings.general, "use-scrollbar")
 
-        # TODO PORT do we need this?
-        # if self.selected_color is None:
-        #    self.selected_color = getattr(self.window.get_style(),
-        #                                  "light")[int(gtk.STATE_SELECTED)]
-        #
-        #    # Reapply the tab color to all button in the tab list, since at least one doesn't have
-        #    # the select color set. This needs to happen AFTER the first show_all, since before,
-        #    # gtk has not loaded the right colors yet.
-        #    for tab in self.tabs.get_children():
-        #        if isinstance(tab, gtk.RadioButton):
-        #            tab.modify_bg(gtk.STATE_ACTIVE, gtk.gdk.Color(str(self.selected_color)))
-
         # move the window even when in fullscreen-mode
         log.debug("Moving window to: %r", window_rect)
         self.window.move(window_rect.x, window_rect.y)
@@ -1375,9 +1409,9 @@ class Guake(SimpleGladeApp):
         """
 
         if self.settings.general.get_boolean('window-losefocus'):
-            self.settings.general.get_boolean('window-losefocus', False)
+            self.settings.general.set_boolean('window-losefocus', False)
         else:
-            self.settings.general.get_boolean('window-losefocus', True)
+            self.settings.general.set_boolean('window-losefocus', True)
         return True
 
     def fullscreen(self):
@@ -1416,7 +1450,7 @@ class Guake(SimpleGladeApp):
         log.debug("Terminal exited: %s", term)
         if libutempter is not None:
             libutempter.utempter_remove_record(term.get_pty())
-            self.delete_tab(self.notebook.page_num(widget), kill=False, prompt=False)
+        self.delete_tab(self.notebook.page_num(widget), kill=False, prompt=False)
 
     def recompute_tabs_titles(self):
         """Updates labels on all tabs. This is required when `self.abbreviate`
@@ -1564,7 +1598,7 @@ class Guake(SimpleGladeApp):
         else:
             text = str.join("", (shell_quote(path) + " " for path in pathlist))
 
-        box.terminal.feed_child(text, len(command))
+        box.terminal.feed_child(text)
         return True
 
     # -- tab related functions --
@@ -1662,10 +1696,12 @@ class Guake(SimpleGladeApp):
         if isinstance(directory, str):
             wd = directory
         else:
-            if self.settings.general.get_boolean('open-tab-cwd'):
-                wd = self.get_current_dir()
-            else:
-                wd = os.environ['HOME']
+            wd = os.environ['HOME']
+            try:
+                if self.settings.general.get_boolean('open-tab-cwd'):
+                    wd = self.get_current_dir()
+            except:  # pylint: disable=bare-except
+                pass
 
         # TODO PORT the next line
         # params['loglastlog'] = login_shell
@@ -1677,7 +1713,11 @@ class Guake(SimpleGladeApp):
         pid = terminal.spawn_sync(
             Vte.PtyFlags.DEFAULT, wd, argv, [], GLib.SpawnFlags.DO_NOT_REAP_CHILD, None, None, None
         )
-        if isinstance(pid, tuple):
+        try:
+            tuple_type = gi._gi.ResultTuple  # pylint: disable=c-extension-no-member
+        except:  # pylint: disable=bare-except
+            tuple_type = tuple
+        if isinstance(pid, (tuple, tuple_type)):
             # Return a tuple in 2.91
             # https://lazka.github.io/pgi-docs/Vte-2.91/classes/Terminal.html#Vte.Terminal.spawn_sync
             pid = pid[1]
@@ -1688,6 +1728,10 @@ class Guake(SimpleGladeApp):
         """This method updates http{s,}_proxy environment variables
         with values found in gconf.
         """
+        if terminal:
+            os.environ['GUAKE_TAB_UUID'] = str(terminal.get_uuid())
+        else:
+            del os.environ['GUAKE_TAB_UUID']
         return
         # TODO PORT port this code
         proxy = '/system/http_proxy/'
@@ -1714,10 +1758,6 @@ class Guake(SimpleGladeApp):
             else:
                 os.environ['http_proxy'] = "http://{!s}:{:d}".format(host, port)
                 os.environ['https_proxy'] = "http://{!s}:{:d}".format(ssl_host, ssl_port)
-        if box:
-            os.environ['GUAKE_TAB_UUID'] = str(terminal.get_uuid())
-        else:
-            del os.environ['GUAKE_TAB_UUID']
 
     def add_tab(self, directory=None):
         """Adds a new tab to the terminal notebook.
@@ -1763,6 +1803,7 @@ class Guake(SimpleGladeApp):
         bnt = Gtk.RadioButton(group=parent, label=label, use_underline=False)
         bnt.set_property('can-focus', False)
         bnt.set_property('draw-indicator', False)
+        bnt.get_style_context().add_class("tab")
         bnt.connect('button-press-event', self.show_tab_menu)
         bnt.connect('button-press-event', self.middle_button_click)
         bnt.connect('button-press-event', self.show_rename_current_tab_dialog)
@@ -1777,9 +1818,6 @@ class Guake(SimpleGladeApp):
         bnt.activate_tab = _update_window_title_on_active_tab
         bnt.connect('clicked', bnt.activate_tab)
 
-        if self.selected_color is not None:
-            # TODO PORT
-            bnt.modify_bg(gtk.STATE_ACTIVE, gtk.gdk.Color(str(self.selected_color)))
         # TODO PORT drag and drop
         # drag_drop_type = ("text/plain", gtk.TARGET_SAME_APP, 80)
         # TODO PORT drag and drop
@@ -2007,15 +2045,33 @@ class Guake(SimpleGladeApp):
                 Gtk.show_uri(self.window.get_screen(), search_url, get_server_time(self.window))
         return True
 
-    def getCurrentTerminalLinkUnderCursor(self):
+    def quick_open(self, *args):
+        """Open open selection
+        """
+        current_term = self.notebook.get_current_terminal()
+
+        if current_term.get_has_selection():
+            self.notebook.get_current_terminal().quick_open()
+        return True
+
+    def get_current_terminal_link_under_cursor(self):
         current_term = self.notebook.get_current_terminal()
         link = current_term.found_link
         log.debug("Current link under cursor: %s", link)
         if link:
             return link
+        return None
+
+    def get_current_terminal_filename_under_cursor(self, selected_text):
+        current_term = self.notebook.get_current_terminal()
+        filename, _1, _2 = current_term.is_file_on_local_server(selected_text)
+        log.info("Current filename under cursor: %s", filename)
+        if filename:
+            return filename
+        return None
 
     def browse_on_web(self, *args):
-        log.debug("browsing %s...", self.getCurrentTerminalLinkUnderCursor())
+        log.debug("browsing %s...", self.get_current_terminal_link_under_cursor())
         self.notebook.get_current_terminal().browse_link_under_cursor()
 
     def set_tab_position(self, *args):
