@@ -44,6 +44,9 @@ from gi.repository import Pango
 from gi.repository import Vte
 
 from guake.common import clamp
+from guake.globals import QUICK_OPEN_MATCHERS
+from guake.globals import TERMINAL_MATCH_EXPRS
+from guake.globals import TERMINAL_MATCH_TAGS
 
 log = logging.getLogger(__name__)
 
@@ -55,25 +58,6 @@ def halt(loc):
 # TODO PORT
 # __all__ = ['Terminal', 'TerminalBox']
 
-# TODO this is not as fancy as as it could be
-# pylint: disable=anomalous-backslash-in-string
-TERMINAL_MATCH_TAGS = ('schema', 'http', 'https', 'email', 'ftp')
-TERMINAL_MATCH_EXPRS = [
-    "(news:|telnet:|nntp:|file:\/|https?:|ftps?:|webcal:)\/\/"
-    "([-[:alnum:]]+(:[-[:alnum:],?;.:\/!%$^*&~\"#']+)?\@)?[-[:alnum:]]+"
-    "(\.[-[:alnum:]]+)*(:[0-9]{1,5})?(\/[-[:alnum:]_$.+!*(),;:@&=?\/~#%]*[^]'.>) \t\r\n,\\\"])?",
-    "(www|ftp)[-[:alnum:]]*\.[-[:alnum:]]+(\.[-[:alnum:]]+)*(:[0-9]{1,5})?"
-    "(\/[-[:alnum:]_$.+!*(),;:@&=?\/~#%]*[^]'.>) \t\r\n,\\\"])?",
-    "(mailto:)?[-[:alnum:]][-[:alnum:].]*@[-[:alnum:]]+\.[-[:alnum:]]+(\\.[-[:alnum:]]+)*"
-]
-# tuple (title/quick matcher/filename and line number extractor)
-QUICK_OPEN_MATCHERS = [(
-    "Python traceback", r"^\s*File\s\".*\",\sline\s[0-9]+", r"^\s*File\s\"(.*)\",\sline\s([0-9]+)"
-), (
-    "line starts by 'Filename:line' pattern (GCC/make). File path should exists.",
-    r"^\s*[a-zA-Z0-9\/\_\-\.\ ]+\.?[a-zA-Z0-9]+\:[0-9]+", r"^\s*.(.*)\:([0-9]+)"
-)]
-
 # pylint: enable=anomalous-backslash-in-string
 
 
@@ -133,6 +117,18 @@ class GuakeTerminal(Vte.Terminal):
     def pid(self, pid):
         self._pid = pid
 
+    def feed_child(self, resolved_cmdline):
+        if (Vte.MAJOR_VERSION, Vte.MINOR_VERSION) >= (0, 42):
+            encoded = resolved_cmdline.encode("utf-8")
+            try:
+                super().feed_child_binary(encoded)
+            except TypeError:
+                # The doc doest not say clearly at which version the feed_child* function has lost
+                # the "len" parameter :(
+                super().feed_child(resolved_cmdline, len(resolved_cmdline))
+        else:
+            super().feed_child(resolved_cmdline, len(resolved_cmdline))
+
     def copy_clipboard(self):
         if self.get_has_selection():
             super(GuakeTerminal, self).copy_clipboard()
@@ -153,13 +149,17 @@ class GuakeTerminal(Vte.Terminal):
             pass
         self.set_audible_bell(client.get_boolean('use-audible-bell'))
         self.set_sensitive(True)
+
+        cursor_blink_mode = self.settings.style.get_int('cursor-blink-mode')
+        self.set_property('cursor-blink-mode', cursor_blink_mode)
+
+        if (Vte.MAJOR_VERSION, Vte.MINOR_VERSION) >= (0, 50):
+            self.set_allow_hyperlink(True)
+
         # TODO PORT there is no method set_flags anymore
         # self.set_flags(gtk.CAN_DEFAULT)
         # self.set_flags(gtk.CAN_FOCUS)
         # TODO PORT getting it and then setting it???
-        # cursor_blink_mode = client.get_int(KEY('/style/cursor_blink_mode'))
-        # client.set_int(KEY('/style/cursor_blink_mode'), cursor_blink_mode)
-        # TODO PORT getting it and then setting it???
         # cursor_shape = client.get_int(KEY('/style/cursor_shape'))
         # client.set_int(KEY('/style/cursor_shape'), cursor_shape)
 
@@ -168,16 +168,33 @@ class GuakeTerminal(Vte.Terminal):
         guake.globals.TERMINAL_MATCH_EXPRS to the terminal to make vte
         highlight text that matches them.
         """
-        # log.debug("Skipped 'match' feature")
-        for expr in TERMINAL_MATCH_EXPRS:
-            # TODO PORT next line throws a Vte-WARNIN but works: runtime check failed
-            tag = self.match_add_gregex(GLib.Regex.new(expr, 0, 0), 0)
-            self.match_set_cursor_type(tag, Gdk.CursorType.HAND2)
+        try:
+            for expr in TERMINAL_MATCH_EXPRS:
+                tag = self.match_add_regex(Vte.Regex.new_for_match(expr, len(expr), 0), 0)
+                self.match_set_cursor_type(tag, Gdk.CursorType.HAND2)
 
-        for _useless, match, _otheruseless in QUICK_OPEN_MATCHERS:
-            # TODO PORT next line throws a Vte-WARNIN but works: runtime check failed
-            tag = self.match_add_gregex(GLib.Regex.new(match, 0, 0), 0)
-            self.match_set_cursor_type(tag, Gdk.CursorType.HAND2)
+            for _useless, match, _otheruseless in QUICK_OPEN_MATCHERS:
+                tag = self.match_add_regex(Vte.Regex.new_for_match(match, len(match), 0), 0)
+                self.match_set_cursor_type(tag, Gdk.CursorType.HAND2)
+        except (GLib.Error, AttributeError) as e:  # pylint: disable=catching-non-exception
+            try:
+                compile_flag = 0
+                if (Vte.MAJOR_VERSION, Vte.MINOR_VERSION) >= (0, 44):
+                    compile_flag = GLib.RegexCompileFlags.MULTILINE
+                for expr in TERMINAL_MATCH_EXPRS:
+                    tag = self.match_add_gregex(GLib.Regex.new(expr, compile_flag, 0), 0)
+                    self.match_set_cursor_type(tag, Gdk.CursorType.HAND2)
+
+                for _useless, match, _otheruseless in QUICK_OPEN_MATCHERS:
+                    tag = self.match_add_gregex(GLib.Regex.new(match, compile_flag, 0), 0)
+                    self.match_set_cursor_type(tag, Gdk.CursorType.HAND2)
+            except GLib.Error as e:  # pylint: disable=catching-non-exception
+                log.error(
+                    "ERROR: PCRE2 does not seems to be enabled on your system. "
+                    "Quick Edit and other Ctrl+click features are disabled. "
+                    "Please update your VTE package or contact your distribution to ask "
+                    "to enable regular expression support in VTE. Exception: '%s'", str(e)
+                )
 
     def get_current_directory(self):
         directory = os.path.expanduser('~')
@@ -187,7 +204,7 @@ class GuakeTerminal(Vte.Terminal):
                 directory = cwd
         return directory
 
-    def _is_file_on_local_server(self, text) -> Tuple[Optional[Path], Optional[int], Optional[int]]:
+    def is_file_on_local_server(self, text) -> Tuple[Optional[Path], Optional[int], Optional[int]]:
         """Test if the provided text matches a file on local server
 
         Supports:
@@ -200,35 +217,61 @@ class GuakeTerminal(Vte.Terminal):
             text (str): candidate for file search
 
         Returns
-            - Tuple(None, None, None) is the provided text does not match anything
+            - Tuple(None, None, None) if the provided text does not match anything
             - Tuple(file path, None, None) if only a file path is found
             - Tuple(file path, linenumber, None) if line number is found
             - Tuple(file path, linenumber, columnnumber) if line and column numbers are found
         """
         lineno = None
         colno = None
+        py_func = None
+        # "<File>:<line>:<col>"
         m = re.compile(r"(.*)\:(\d+)\:(\d+)$").match(text)
         if m:
             text = m.group(1)
             lineno = m.group(2)
             colno = m.group(3)
         else:
+            # "<File>:<line>"
             m = re.compile(r"(.*)\:(\d+)$").match(text)
             if m:
                 text = m.group(1)
                 lineno = m.group(2)
+            else:
+                # "<File>::<python_function>"
+                m = re.compile(r"^(.*)\:\:([a-zA-Z0-9\_]+)$").match(text)
+                if m:
+                    text = m.group(1)
+                    py_func = m.group(2).strip()
+
+        def find_lineno(text, pt, lineno, py_func):
+            # print("text={!r}, pt={!r}, lineno={!r}, py_func={!r}".format(text,
+            #                                                              pt, lineno, py_func))
+            if lineno:
+                return lineno
+            if not py_func:
+                return
+            with pt.open() as f:
+                for i, line in enumerate(f.readlines()):
+                    if line.startswith("def {}".format(py_func)):
+                        return i + 1
+                        break
+
         pt = Path(text)
         log.debug("checking file existance: %r", pt)
         if pt.exists():
-            log.debug("File exists: %r", pt.absolute().as_posix())
+            lineno = find_lineno(text, pt, lineno, py_func)
+            log.info("File exists: %r, line=%r", pt.absolute().as_posix(), lineno)
             return (pt, lineno, colno)
         log.debug("No file found matching: %r", text)
         cwd = self.get_current_directory()
         pt = Path(cwd) / pt
         log.debug("checking file existance: %r", pt)
         if pt.exists():
-            log.debug("File exists: %r", pt.absolute().as_posix())
+            lineno = find_lineno(text, pt, lineno, py_func)
+            log.info("File exists: %r, line=%r", pt.absolute().as_posix(), lineno)
             return (pt, lineno, colno)
+        log.info("file does not exist: %s", str(pt))
         return (None, None, None)
 
     def button_press(self, terminal, event):
@@ -237,96 +280,91 @@ class GuakeTerminal(Vte.Terminal):
         handle the matched resource uri.
         """
         self.matched_value = ''
-        matched_string = self.match_check(
-            int(event.x / self.get_char_width()), int(event.y / self.get_char_height())
-        )
+        if (Vte.MAJOR_VERSION, Vte.MINOR_VERSION) >= (0, 46):
+            matched_string = self.match_check_event(event)
+        else:
+            matched_string = self.match_check(
+                int(event.x / self.get_char_width()), int(event.y / self.get_char_height())
+            )
 
         self.found_link = None
 
         if event.button == 1 and (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-            if self.get_has_selection():
-                self.copy_clipboard()
-                clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
-                text = clipboard.wait_for_text()
-                (fp, lo, co) = self._is_file_on_local_server(text)
-                self._execute_quick_open(fp, lo)
+            if (Vte.MAJOR_VERSION, Vte.MINOR_VERSION) > (0, 50):
+                s = self.hyperlink_check_event(event)
+                self._on_ctrl_click_matcher((s, None))
+            elif self.get_has_selection():
+                self.quick_open()
             elif matched_string and matched_string[0]:
-                self.__on_ctrl_click_matcher(matched_string)
+                self._on_ctrl_click_matcher(matched_string)
         elif event.button == 3 and matched_string:
             self.found_link = self.handleTerminalMatch(matched_string)
             self.matched_value = matched_string[0]
 
-    def __on_ctrl_click_matcher(self, matched_string):
+    def quick_open(self):
+        self.copy_clipboard()
+        clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+        text = clipboard.wait_for_text()
+        if not text:
+            return
+        (fp, lo, co) = self.is_file_on_local_server(text)
+        self._execute_quick_open(fp, lo)
+
+    def _on_ctrl_click_matcher(self, matched_string):
         value, tag = matched_string
+        found_matcher = False
         log.debug("matched string: %s", matched_string)
         # First searching in additional matchers
-        found_additional_matcher = False
-        # TODO PORT
         use_quick_open = self.settings.general.get_boolean("quick-open-enable")
-        quick_open_in_current_terminal = self.settings.general.get_boolean(
-            "quick-open-in-current-terminal"
-        )
         if use_quick_open:
-            for _useless, _otheruseless, extractor in QUICK_OPEN_MATCHERS:
-                print("value", value)
-                g = re.compile(extractor).match(value)
-                if g and g.groups():
-                    filename = g.group(1).strip()
-                    filepath = filename
-                    line_number = g.group(2)
-                    if line_number is None:
-                        line_number = "1"
-                    if not quick_open_in_current_terminal:
-                        curdir = self.get_current_directory()
-                        filepath = os.path.join(curdir, filename)
-                        filepaths = [filepath]
-                        tmp_filepath = filepath
-                        # Also check files patterns that ends with one or 2 ':'
-                        for _ in range(2):
-                            if ':' not in tmp_filepath:
-                                break
-                            tmp_filepath = tmp_filepath.rpartition(':')[0]
-                            filepaths.append(tmp_filepath)
-                        log.debug("Testing existance of the following files: %r", filepaths)
-                        for filepath in filepaths:
-                            if os.path.exists(filepath):
-                                break
-                        else:
-                            logging.info(
-                                "Cannot open file %s, it doesn't exists locally"
-                                "(current dir: %s)", filepath, os.path.curdir
-                            )
-                            log.debug("No file exist")
-                            continue
-                    # for quick_open_in_current_terminal, we run the command line directly in
-                    # the tab so relative path is enough.
-                    #
-                    # We do not test for file existence, because it doesn't work in ssh
-                    # sessions.
-                    self._execute_quick_open(filepath, line_number)
-                    found_additional_matcher = True
-                    break
-        if not found_additional_matcher:
+            found_matcher = self._find_quick_matcher(value)
+        if not found_matcher:
             self.found_link = self.handleTerminalMatch(matched_string)
             if self.found_link:
                 self.browse_link_under_cursor()
 
+    def _find_quick_matcher(self, value):
+        for _useless, _otheruseless, extractor in QUICK_OPEN_MATCHERS:
+            g = re.compile(extractor).match(value)
+            if g and g.groups():
+                filename = g.group(1).strip()
+                if len(g.groups()) >= 2:
+                    line_number = g.group(2)
+                else:
+                    line_number = None
+                log.info("Quick action executed filename=%s, line=%s", filename, line_number)
+                (filepath, ln, _) = self.is_file_on_local_server(filename)
+                if ln:
+                    line_number = ln
+                if not filepath:
+                    continue
+                if line_number is None:
+                    line_number = "1"
+                self._execute_quick_open(filepath, line_number)
+                return True
+        return False
+
     def _execute_quick_open(self, filepath, line_number):
+        if not filepath:
+            return
         cmdline = self.settings.general.get_string("quick-open-command-line")
-        quick_open_in_current_terminal = self.settings.general.get_boolean(
-            "quick-open-in-current-terminal"
-        )
         if not line_number:
             line_number = ""
+        else:
+            line_number = str(line_number)
         logging.debug("Opening file %s at line %s", filepath, line_number)
         resolved_cmdline = cmdline % {"file_path": filepath, "line_number": line_number}
         logging.debug("Command line: %s", resolved_cmdline)
+        quick_open_in_current_terminal = self.settings.general.get_boolean(
+            "quick-open-in-current-terminal"
+        )
         if quick_open_in_current_terminal:
             logging.debug("Executing it in current tab")
             if resolved_cmdline[-1] != '\n':
                 resolved_cmdline += '\n'
-            self.feed_child(resolved_cmdline, len(resolved_cmdline))
+            self.feed_child(resolved_cmdline)
         else:
+            resolved_cmdline += " &"
             logging.debug("Executing it independently")
             subprocess.call(resolved_cmdline, shell=True)
 
@@ -396,7 +434,7 @@ class GuakeTerminal(Vte.Terminal):
         UI, so you can use python's start_new_thread.
         """
         try:
-            os.kill(pid.child_pid, signal.SIGHUP)
+            os.kill(pid, signal.SIGHUP)
         except OSError:
             pass
         num_tries = 30
@@ -406,7 +444,7 @@ class GuakeTerminal(Vte.Terminal):
                 # Try to wait for the pid to be closed. If it does not
                 # exist anymore, an OSError is raised and we can
                 # safely ignore it.
-                if os.waitpid(pid.child_pid, os.WNOHANG)[0] != 0:
+                if os.waitpid(pid, os.WNOHANG)[0] != 0:
                     break
             except OSError:
                 break
@@ -415,8 +453,8 @@ class GuakeTerminal(Vte.Terminal):
 
         if num_tries == 0:
             try:
-                os.kill(pid.child_pid, signal.SIGKILL)
-                os.waitpid(pid.child_pid, 0)
+                os.kill(pid, signal.SIGKILL)
+                os.waitpid(pid, 0)
             except OSError:
                 # if this part of code was reached, means that SIGTERM
                 # did the work and SIGKILL wasnt needed.
